import sys
from argparse import ArgumentParser

import requests

from ..utils import get_code


def set_infos_parser(parser: ArgumentParser):
    parser.add_argument("ide", nargs="?", default="all")


def infos_cmd(args, config_fttb):
    ide = get_code(args.ide, config_fttb)
    res = requests.get(
        f"https://data.services.jetbrains.com/products?"
        f"fields=name,intellijProductCode,description,categories&"
        f"code={ide}"
    )

    if not res.ok:
        print("request failed")
        sys.exit()

    if len(res.json()) == 0:
        return
    ide = res.json()[0]

    print(f"{ide['name']}\ncode: {ide['intellijProductCode']}\n{ide['description']}\n")



import glob
import os
import shutil
import sys
from argparse import ArgumentParser

import requests

from ..utils import get_code, parse_version


def set_remove_parser(parser: ArgumentParser):
    parser.add_argument("ide", nargs="?", default="all")
    parser.add_argument("version", nargs='?', default="all")
    parser.add_argument("--type", choices=["release", "eap", "rc"], default="release")


def remove_all_versions(args, config_fttb):
    ide_code = get_code(args.ide, config_fttb)
    for f in glob.glob(f"{config_fttb['install_path']}/{ide_code}-*"):
        shutil.rmtree(f)


def remove_cmd(args, config_fttb):
    if args.ide == "all":
        print("invalid IDE code")
        return
    if args.version == "all":
        return remove_all_versions(args, config_fttb)
    ide_code = get_code(args.ide, config_fttb)
    
    res = requests.get(
        f"https://data.services.jetbrains.com/products?code={ide_code}&fields=releases")
    if not res.ok:
        print("request failed")
        sys.exit()

    version = parse_version(ide_code, args.version, args.type)

    try:
        shutil.rmtree(f"{config_fttb['install_path']}/{ide_code}-{version}")
    except FileNotFoundError:
        pass
    if not os.path.exists(f"{config_fttb['bin_path']}/{args.ide}"):
        try:
            os.remove(f"{config_fttb['bin_path']}/{args.ide}")
        except FileNotFoundError:
            pass



import os
from argparse import ArgumentParser

import requests

from ..cmds.download import download_ide
from ..utils import get_code, parse_version, VersionError


def set_use_parser(parser: ArgumentParser):
    parser.add_argument("ide", nargs="?", default="all")
    parser.add_argument("version", nargs='?', default="latest")
    parser.add_argument("--type", dest="type", choices=["release", "eap", "rc"], default="release")


def is_used(ide, ide_code, version, config_fttb):
    if not os.path.exists(f"{config_fttb['bin_path']}/{ide}") or \
            not os.path.exists(f"{os.getenv('HOME')}/.local/share/applications/{ide}.desktop"):
        return False
    if not os.path.realpath(
            f"{config_fttb['bin_path']}/{ide}") != f"{config_fttb['install_path']}/{ide_code}-{version}/bin/{ide}.sh":
        return False
    with open(f"{os.getenv('HOME')}/.local/share/applications/{ide}.desktop", "r") as entry_file:
        for line in entry_file:
            if line.startswith("Exec="):
                if version not in line:
                    return False
    return True


def generate_entry(ide, ide_code, version, config_fttb):
    if is_used(ide, ide_code, version, config_fttb):
        return
    res = requests.get(
        "https://data.services.jetbrains.com/products?"
        f"code={ide_code}&"
        "fields=name,intellijProductCode,description,categories")

    template_file_res = requests.get(
        "https://gist.githubusercontent.com/BenoitRoux0/ece685d71749e9d52a1c03b09a5b6e74/raw"
        "/dd5bd0f2a2f24c157a26aa7c97121f883dd6eeef/template.desktop")
    entry = template_file_res.content.decode()
    entry = entry.replace(
        "{name}",
        res.json()[0]['name']
    )
    entry = entry.replace(
        "{desc}",
        res.json()[0]['description']
    )
    entry = entry.replace(
        "{exec}",
        f"{config_fttb['install_path']}/{ide_code}-{version}/bin/{ide} %U"
    )
    entry = entry.replace(
        "{icon}",
        f"{config_fttb['install_path']}/{ide_code}-{version}/bin/{ide}.svg"
    )
    entry_file = open(f"{os.getenv('HOME')}/.local/share/applications/{ide}.desktop", "w+")
    entry_file.write(entry)
    entry_file.close()
    try:
        os.remove(f"{config_fttb['bin_path']}/{ide}")
    except FileNotFoundError:
        pass
    os.symlink(f"{config_fttb['install_path']}/{ide_code}-{version}/bin/{ide}", f"{config_fttb['bin_path']}/{ide}")


def use_cmd(args, config_fttb):
    ide_code = get_code(args.ide, config_fttb)
    try:
        version = parse_version(ide_code, args.version, args.type)
    except VersionError:
        print("bad version")
        return
    download_ide(ide_code, version, args.type, config_fttb)
    generate_entry(args.ide, ide_code, version, config_fttb)



import os

from ..utils import get_versions_list, get_all_versions


def update_cmd():
    ides = get_versions_list()
    for ide_code in ides:
        try:
            os.remove(f"/tmp/ft_tb/{ide_code}-versions.json")
        except FileNotFoundError:
            pass
        print(f"update {ide_code}")
        get_all_versions(ide_code)



import json
import os
import sys

import requests


class VersionError(Exception):
    pass


def get_versions_list() -> list[str]:
    with open("/tmp/ft_tb/versions.list", "r") as list_file:
        return list_file.read().splitlines()


def save_versions_list(ide_code):
    with open("/tmp/ft_tb/versions.list", "a") as _:
        os.chmod("/tmp/ft_tb/versions.list", 0o777)
    with open("/tmp/ft_tb/versions.list", "r") as list_file:
        ides = list_file.read().splitlines()
    if ide_code in ides:
        return
    ides.append(ide_code)
    with open("/tmp/ft_tb/versions.list", "w") as list_file:
        list_file.write("\n".join(ides))


def get_all_versions(ide_code):
    versions_list: dict
    if os.path.exists(f"/tmp/ft_tb/{ide_code}-versions.json"):
        os.chmod(f"/tmp/ft_tb/{ide_code}-versions.json", 0o777)
        with open(f"/tmp/ft_tb/{ide_code}-versions.json", "r") as cache_file:
            save_versions_list(ide_code)
            return json.load(cache_file)

    res = requests.get(
        f"https://data.services.jetbrains.com/products?code={ide_code}&fields=releases")
    if not res.ok:
        print("request failed")
        sys.exit()
    releases = res.json()[0]['releases']
    with open(f"/tmp/ft_tb/{ide_code}-versions.json", "w+") as cache_file:
        os.chmod(f"/tmp/ft_tb/{ide_code}-versions.json", 0o777)
        json.dump(releases, cache_file)
    save_versions_list(ide_code)
    return releases


def get_latest(version_type: str | None, releases):
    for release in releases:
        if version_type is None or release["type"] == version_type:
            return release['version']
    raise VersionError


def parse_version(ide_code, version, version_type: str | None):
    releases = get_all_versions(ide_code)
    if version == "latest":
        return get_latest(version_type, releases)
    for release in releases:
        if release['version'] == version and (version_type is None or release["type"] == version_type):
            return version
    raise VersionError


def open_config(path):
    file = open(path, "r")
    conf = json.load(file)
    return conf


def get_code(name, config_fttp: dict):
    if name in config_fttp.values():
        return name
    if name in config_fttp['aliases'].keys():
        return config_fttp['aliases'][name]
    return name


def download_file(url, dst):
    with open(dst, "wb") as f:
        response = requests.get(url, stream=True)
        total_length = response.headers.get('content-length')

        if total_length is None:
            f.write(response.content)
        else:
            dl = 0
            total_length = int(total_length)
            for data in response.iter_content(chunk_size=4096):
                dl += len(data)
                f.write(data)
                done = int(100 * dl / total_length)
                print(f"{done}%", end="\r")
                sys.stdout.flush()
            print("")


def create_config():
    config_fttb: dict
    try:
        os.makedirs(f"{os.getenv('HOME')}/.config/fttb")
    except FileExistsError:
        pass
    if not os.path.exists(f"{os.getenv('HOME')}/.config/fttb/config.json"):
        download_file(
            "https://gist.githubusercontent.com/BenoitRoux0/16b18e10cfd53dcf31a28cb1b38e4303/raw"
            "/676f61b08f669dffda5f60a9f54fb3cc1eb18542/config.json",
            f"{os.getenv('HOME')}/.config/fttb/config.json")
        config_fttb = open_config(f"{os.getenv('HOME')}/.config/fttb/config.json")
        config_fttb['cache_path'] = f"{os.getenv('HOME')}/.cache/fttb"
        config_fttb['install_path'] = f"{os.getenv('HOME')}/goinfre/ides/fttb"
        config_fttb['bin_path'] = f"{os.getenv('HOME')}/bin"
        with open(f"{os.getenv('HOME')}/.config/fttb/config.json", "w") as conf_file:
            json.dump(config_fttb, conf_file)
        print(config_fttb)
    else:
        config_fttb = open_config(f"{os.getenv('HOME')}/.config/fttb/config.json")
    try:
        os.makedirs(config_fttb['bin_path'])
    except FileExistsError:
        pass
    try:
        os.makedirs(config_fttb['cache_path'])
    except FileExistsError:
        pass
    try:
        os.makedirs(config_fttb['install_path'])
    except FileExistsError:
        pass
    try:
        os.makedirs("/tmp/ft_tb")
    except FileExistsError:
        pass
    os.chmod("/tmp/ft_tb", 0o777)
